/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// created by : r.nagel 28.04.2005
//
// function : handling of all available language versions (from properties file set)
//            extends AbstractTableModel for Language-Managing-Dialog
//
// todo     :
//
// modified :

package net.sf.langproper.engine ;

import java.util.Vector;
import java.util.logging.Logger;
import javax.swing.table.*;


public class TLanguageList extends AbstractTableModel
{

  public static final int
      KEY_VALUE      = -3,
      UNDEFINED_LANG = -2,
      DEFAULT_LANG = -1 ;

  /** information about language versions (single properties file) */
  private Vector langVersions ;

  /** placeholder for a default translation */
  private TLanguageVersionFile defaultLang ;

  /** visibility of single data changed */
  private boolean dataVisibilityChanged ;

  private int visibleVersionsCount ;

  //----------------------------------------------------------------------------

  public TLanguageList()
  {
    langVersions = new Vector( 5 ) ;
    dataVisibilityChanged = false ;
    visibleVersionsCount = -1 ;
  }

  //----------------------------------------------------------------------------

  /** clear all data */
  public void clear()
  {
    langVersions.clear();
    defaultLang = null ;
    dataVisibilityChanged = false ;
    visibleVersionsCount = -1 ;
  }

  /** returns the number of available language versions - without default language */
  public int size()
  {
    return langVersions.size() ;
  }

  /** returns the number of ALL visible language versions - without default language */
  public int getVisibleSize()
  {
    if ( (visibleVersionsCount < 0) || dataVisibilityChanged)
    {
      int len = langVersions.size() ;
      int vLen = 0 ;

      // count all visible languages
      for ( int t = 0 ; t < len ; t++ )
      {
        if ( get( t ).isVisible() )
          vLen++ ;
      }
      return vLen ;
    }

    return visibleVersionsCount ;
  }

  /** returns the language with ID <langID> */
  public TLanguageVersionFile get(int langID)
  {
    return ( TLanguageVersionFile ) langVersions.get( langID ) ;
  }

  //----------------------------------------------------------------------------

  /** put a languagefile into array */
  public void addLanguageVersion( TLanguageVersionFile versionFile )
  {
    if ( versionFile.hasLanguageExtension() )
    {
      int index = getLanguageID( versionFile.getVersion() ) ;
      if ( index == UNDEFINED_LANG ) // not found
      {
        langVersions.add( versionFile ) ;
      }
      else
      {
        Logger.global.fine( "language version already defined " +
                             versionFile.getFile() ) ;
      }
    }
    else // no Extension
    {
      if ( defaultLang == null )
      {
        defaultLang = versionFile ;
      }
      else
      {
        Logger.global.fine( "default version already defined " +
                            versionFile.getFile() ) ;
      }
    }
  }

  //----------------------------------------------------------------------------

  /** returns the internal ID of the language id-string */
  public int getLanguageID( String lang )
  {
    int back = UNDEFINED_LANG ;

    if ( lang != null )
    {
      if ( lang.length() > 0 )
      {
        boolean found = false ;
        int t = 0 ;
        int len = langVersions.size() ;
        while ( ( !found ) && ( t < len ) )
        {
          TLanguageVersionFile vf = ( TLanguageVersionFile ) langVersions.get(
              t ) ;
          if ( vf != null )
          {
            String id = vf.getVersion() ;
            if ( id.hashCode() == lang.hashCode() )
            {
              found = true ;
              back = t ;
            }
            else
            {
              t++ ;
            }
          }
        }
      }
      else
      {
        back = DEFAULT_LANG ;
      }
    }
    return back ;
  }

  //----------------------------------------------------------------------------
  // AbstractTableModel
  // ---------------------------------------------------------------------------


  public int getColumnCount()
  {
    return 3 ;
  }

  public String getColumnName( int col )
  {
    switch (col)
    {
      case 0 : return "iso code";
      case 1 : return "language" ;
      case 2 : return "visible" ;
    }
    return "-" ;
  }

  public Class getColumnClass( int c )
  {
    if (c == 2)
      return Boolean.class ;

    return String.class ;
  }

  public boolean isCellEditable( int rowIndex, int columnIndex)
  {
    if (columnIndex == 2)  // visible/invisible checkbox
    {
      // default entry can't changed to invisible
      if ((rowIndex == 0) && (defaultLang != null))
      {
        return false ;
      }
      return true ;
    }
    return false ;
  }


  public int getRowCount()
  {
    int back = langVersions.size() ;
    if ( defaultLang != null)
      back++ ;

    return back ;
  }

  // returns the LanguageVersionFile Object for row <row>
  private TLanguageVersionFile getData(int row)
  {
    TLanguageVersionFile dummy = null ;
    if ((row == 0) && (defaultLang != null))
    {
      dummy = defaultLang ;
    } else if (defaultLang != null)
    {
      dummy = (TLanguageVersionFile) langVersions.get(row -1) ;
    }
    else
    {
      dummy = (TLanguageVersionFile) langVersions.get(row) ;
    }

    return dummy ;
  }

  public Object getValueAt( int rowIndex, int columnIndex )
  {
    Object back = "" ;
    TLanguageVersionFile dummy = getData(rowIndex) ;

    if (dummy != null)
    {
       switch (columnIndex)
       {
         case 2 :  // visibility
           back = dummy.getVisible() ;
           break ;
         case 1 : // language name
           back = dummy.getFullLanguageName() ;
           break ;
         default :  // language iso code
           if ( (defaultLang != null) && (rowIndex == 0))
             back = "default" ;
           else back = dummy.getLanguageCode() ;
       }
    }

    return  back ;
  }

  public void setValueAt( Object aValue, int rowIndex, int columnIndex )
  {
//    System.out.println( "set " + rowIndex + ", " + columnIndex ) ;
    TLanguageVersionFile dummy = getData(rowIndex) ;
    if (dummy != null)
    {
      if (aValue != null)
      {
        Boolean data = (Boolean) aValue ;
        dummy.setVisible( data.booleanValue());
        dataVisibilityChanged = true ;
      }
    }
  }

  /** the visibility of some language versions was changed */
  public boolean hasDataVisibiltyChanged()
  {
    return dataVisibilityChanged ;
  }

  /** resets all flags for data (dataVisibiltyChanged) */
  public void resetDataFlags()
  {
    dataVisibilityChanged = false ;
  }


  //----------------------------------------------------------------------------
  // handling of optional default language version
  // ---------------------------------------------------------------------------
  public boolean hasDefaultLang()
  {
    if (defaultLang == null)
      return false ;

    return true ;
  }
  public TLanguageVersionFile getDefaultLang()
  {
    return defaultLang;
  }

  public void setDefaultLang(TLanguageVersionFile pDefaultLang)
  {
    this.defaultLang = pDefaultLang;
  }
  //----------------------------------------------------------------------------


  /** sorts all entries - now only the key sorting is implemented
   *  and the langID is ignored */
  public void sortElements()
  {
    Object eList[] = langVersions.toArray() ;
    java.util.Arrays.sort(eList, new TLanguageNameComparator() );

       langVersions.clear();
      for (int t = 0 ; t < eList.length ; t++)
      {
         langVersions.add(eList[t]) ;
      }
  }

  // -------------------------------------------------------------------------
  class TLanguageNameComparator implements java.util.Comparator
  {
    public final int compare(Object o1, Object o2)
    {
      TLanguageVersionFile p1 = null ;
      TLanguageVersionFile p2 = null ;

      try
      {
        p1 = (TLanguageVersionFile) o1 ;
        p2 = (TLanguageVersionFile) o2 ;
      }
      catch (Exception e) {}

      int back = 0 ;

      if ((p1 != null) && (p2 != null))
      {
        back = compareTo(p1.getLanguageCode(), p2.getLanguageCode()) ;
        if (back == 0)
          back = compareTo(p1.getCountryCode(), p2.getCountryCode()) ;
      }

      return back ;
    }

    public final boolean equals(Object obj)
    {
      return false ;
    }

    private int compareTo(String s1, String s2)
    {
      if (s1 == null)
        s1 = "" ;

      if (s2 == null)
        s2 = "" ;
      return s1.compareTo(s2) ;
    }

  }

}
