/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// created by : r.nagel 4.4.2005
//
// function : contains a key and all appendant translations (and comments)
//
//            some words about comments handling :
//            every value (key specific translation) can have comments
//            -> for every translation there is a comments object
//
// todo     :
//
//
// modified :


package net.sf.langproper.engine ;

public class TMultiLanguageEntry
{

  // used for a translation
  private class TValueEntry
  {
    private String value ;
    private TComments comment ;  // every key can have some comments

    public TValueEntry(String pValue, TComments pComment)
    {
      value = pValue ;
      comment = pComment ;
    }

    public void set(String pValue, TComments pComment)
    {
      value = pValue ;
      // !!!!!!!!!!! add or replace the comment ?
      if (pComment != null)
        comment = pComment ;
    }
  }

  public static final int
      STATE_UNCHANGED  = 2,
      STATE_MODIFIED   = 5
      ;

  private String keyData ;

  private TValueEntry defValue ;
  private TValueEntry values[] ;

  private int status ;

  private int len ;

  private boolean hasAnyComment ;

  /** creates a new Object with <size> empty translations */
  public TMultiLanguageEntry(int size)
  {
    status = STATE_UNCHANGED ;

    values = new TValueEntry[size] ;
    len = size ;
    hasAnyComment = false ;
  }

  // --------------------------------------------------------------------------

  private TValueEntry getValueEntry(int langID)
  {
    TValueEntry back ;
    if ( (langID >=  0) && (langID < len))
    {
      back = values[langID] ;
    }
    else back = defValue ;

    return back ;
  }

  // --------------------------------------------------------------------------

  public void setTranslation(String value, int langID)
  {
    setTranslation(value, null, langID) ;
  }

  /** Set a new value for translation with ID <langID>
   *  If the param <comment> is null, the old comment isn't affected. Otherwise
   *  the old comment is overwritten.
   */
  public void setTranslation(String value, TComments comment, int langID)
  {
    if (langID < 0)
    {
      if (defValue == null)
        defValue = new TValueEntry(value, comment) ;
      else
      {
        defValue.set(value, comment);
      }
    }
    else if (langID < len)
    {
      TValueEntry dummy = values[langID] ;
      if (dummy == null)
      {
        values[langID] = new TValueEntry(value, comment) ;
      }
      else
      {
        dummy.set(value, comment);
      }
    }
    if (comment != null)
      hasAnyComment = true ;
  }

  /** returns the language dependend translation or if it is not available the
   *  default translation or the key value
   */
  public String getTranslation(int langID)
  {
    if ( (langID >=  0) && (langID < len))
    {
      TValueEntry dummy = values[langID] ;
      if (dummy != null)
        return dummy.value ;
    }

    // no translation but a default value
    if (defValue != null)
      return defValue.value ;

    // no default and translation found
    return this.getKey() ;
  }

  /** returns a translation or null if nothing was found */
  public String getTranslationOnly(int langID)
  {
    if ( (langID >=  0) && (langID < len))
    {
      TValueEntry dummy = values[langID] ;
      if (dummy != null)
        return dummy.value ;
    }
    else if (TLanguageList.DEFAULT_LANG == langID)
    {
        // no translation but a default value
        if (defValue != null)
          return defValue.value ;
    }

    return null ;
  }


  // --------------------------------------------------------------------------

  /** set the comments for translation */
  public void setComments(TComments comment, int langID)
  {
    TValueEntry entry = getValueEntry(langID) ;
    if (entry != null)
    {
      entry.comment = comment ;
    }
  }

  /** get all comments for translation */
  public TComments getComments(int langID)
  {
    TValueEntry entry = getValueEntry(langID) ;
    if (entry != null)
    {
      return entry.comment ;
    }

    return null ;
  }

  /** returns true, if the translation (langID) contains a comment */
  public boolean hasComment(int langID)
  {
    TValueEntry entry = getValueEntry(langID) ;
    if (entry != null)
    {
      if (entry.comment != null)
        return true ;
    }

    return false ;
  }

  /** returns true, if any translation contains a comment */
  public boolean hasComment()
  {
  return hasAnyComment ;
  }

  // --------------------------------------------------------------------------
  public String getKey()
  {
    return keyData;
  }

  public void setKey(String pKey)
  {
    this.keyData = pKey ;
  }

  // --------------------------------------------------------------------------

  public int getStatus()
  {
    return status;
  }

  public void setStatus(int pStatus)
  {
    this.status = pStatus;
  }

  // --------------------------------------------------------------------------

  /** returns true if all language translation entries are available */
  public boolean isComplete()
  {
    boolean back = true ;

    int t = 0;
    while ( (t < len) && back)
    {
      if (values[t] == null)
      {
        return false ;
      }

      String str = values[t].value ;
      if (str == null)
      {
        back = false ;
      }
      else if (str.length() < 1)
      {
        back = false ;
      }
      else t++ ;
    }

    return back ;
  }

  /** returns the number of empty translations */
  public int getEmptyTranslationCount()
  {
    int back = 0 ;

    for(int t = 0 ; t < len ; t++)
    {
      TValueEntry dummy = values[t] ;
      if (dummy != null)
      {
        String str = dummy.value ;
        if ( str == null )
        {
          back++ ;
        }
        else if ( str.length() < 1 )
        {
          back++ ;
        }
      } else back++ ; // no TValueEntry
    }

    return back ;
  }
}
