/*
Popeye - Java (Language) Properties File Editor

Copyright (C) 2005 Raik Nagel <kiar@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* Neither the name of the author nor the names of its contributors may be
  used to endorse or promote products derived from this software without
  specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package net.sf.langproper.gui.theme ;


import javax.swing.plaf.metal.*;
import javax.swing.plaf.basic.BasicBorders;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;

/**
* Defines all themes which can allow the user to customize the Java Look and Feel.
*
* <P>This application uses only the cross-platform Java Look-and-Feel, and never
* attempts to adapt to the native look-and-feel (Windows, Metal, Mac).
*
* @used.By {@link GeneralLookPreferencesEditor}, {@link StocksMonitorMainWindow}.
* @author <a href="http://www.javapractices.com/">javapractices.com</a>
*/
public final class SimpleTheme
{
  /**
  * A theme identical to the default Java look-and-feel, but whose name
  * is "Default" instead of the cryptic "Steel", and which provides a
  * <code>toString</code> method (required if <code>Theme</code> objects
  * passed to a combo box). Used as the base class for all other themes
  * used in this application.
  */
  public static final MetalTheme DEFAULT =  new Default();

  /**
  * Much like {@link #DEFAULT}, but uses some blue-green colors.
  */
  public static final MetalTheme AQUA =  new Aqua();

  /**
  * Much like {@link #DEFAULT}, but uses some other colors and fonts.
  */
  public static final MetalTheme MOON =  new Moon();


  /**
  * Differs from {@link #DEFAULT} only in font sizes.
  */
  public static final MetalTheme LARGE_FONT =  new LargeFont();

  /**
  * Large fonts, and high contrast black and white colors.
  *
  * <P>This is an amalgam of two example themes from the JDK swing examples ; there
  * is apparently no recommended standard for a low-vision theme.
  */
  public static final MetalTheme LOW_VISION =  new LowVision();

  /**
  * Convert <code>aText</code> into its corresponding <code>Theme</code> object,
  * if possible.
  *
  * @param aText possibly-null text which may map to a Theme.
  * @return null if <code>aText</code> is null, else try to match to a
  * known <code>Theme</code>Theme.
  * @throws IllegalArgumentException if <code>aText</code> cannot be
  * matched to a known theme.
  */
  public static MetalTheme valueOf(String aText) {
    if (aText == null) return null;
    Iterator valuesIter = VALUES.iterator();
    MetalTheme theme = null;
    while( valuesIter.hasNext() ) {
      theme = (MetalTheme)valuesIter.next();
      if ( aText.equals( theme.getName() ) ) {
        return theme;
      }
    }
    throw new IllegalArgumentException("Cannot parse into Theme object:" + aText);
  }

  /**
  * Return true if <code>aTheme</code> uses a larger font than the default; this is the
  * case only for <code>LARGE_FONT</code> and <code>LOW_VISION</code>.
  *
  * Themes with large font sizes need particular care, as their use may
  * require changes outside those provided through
  * <code>SwingUtilities.updateComponentTreeUI</code>.
  */
  static boolean hasLargeFont(MetalTheme aTheme) {
    return aTheme == LARGE_FONT || aTheme == LOW_VISION;
  }

  private static final MetalTheme[] fValues = {
    DEFAULT,
    AQUA,
    LARGE_FONT,
    LOW_VISION
  };

  /**
  * Allows user to iterate over all elements of this enumeration.
  */
  public static final java.util.List VALUES =
                                     Collections.unmodifiableList(Arrays.asList(fValues));

  private static final String fDEFAULT_NAME = "Default";
  private static final String fAQUA_NAME = "Aqua";
  private static final String fLARGE_FONT_NAME = "Large Font";
  private static final String fLOW_VISION_NAME = "Low Vision";
  private static final String fMOON_NAME = "Low Vision";

  /*
  * All items below are private nested classes which define the various
  * themes.
  */

  private static class Default extends DefaultMetalTheme {
    public String getName(){
      return fName;
    }
    /**
    * This override is provided such that Theme objects can
    * be directly passed to JComboBox, instead of Strings. (This would
    * not be necessary if getName had been named toString instead).
    */
    public final String toString() {
      return getName();
    }
    private final String fName = fDEFAULT_NAME;
  }

// ---------------------------------------------------------------------------

  private static class Moon extends Default
  {
    public String getName(){ return fName; }

    protected ColorUIResource getPrimary1() { return fPrimary1; }
    protected ColorUIResource getPrimary2() { return fPrimary2; }
    protected ColorUIResource getPrimary3() { return fPrimary3; }

    private final String fName = fMOON_NAME;
//    private final ColorUIResource fPrimary1 = new ColorUIResource(102, 153, 153);
//    private final ColorUIResource fPrimary1 = new ColorUIResource(0x98ACD4);
    private final ColorUIResource fPrimary1 = new ColorUIResource(0x0D3079);

//    private final ColorUIResource fPrimary2 = new ColorUIResource(128, 192, 192); // 0x80C0C0
    private final ColorUIResource fPrimary2 = new ColorUIResource(0x1A469F);

//    private final ColorUIResource fPrimary3 = new ColorUIResource(0xC41C30);
    private final ColorUIResource fPrimary3 = new ColorUIResource(0xB0C4ED); // 0x9FEBEB
//    private final ColorUIResource fPrimary3 = new ColorUIResource(159, 235, 235); // 0x9FEBEB

    //fonts are larger than defaults
    public FontUIResource getControlTextFont() { return fControlFont;}
    public FontUIResource getSystemTextFont() { return fSystemFont;}
    public FontUIResource getUserTextFont() { return fUserFont;}
    public FontUIResource getMenuTextFont() { return fControlFont;}
    public FontUIResource getWindowTitleFont() { return fWindowTitleFont;}
    public FontUIResource getSubTextFont() { return fSmallFont;}

    private final FontUIResource fControlFont = new FontUIResource("Arial", Font.PLAIN, 12);
    private final FontUIResource fSystemFont = new FontUIResource("Arial", Font.PLAIN, 12);
    private final FontUIResource fWindowTitleFont = new FontUIResource("Arial", Font.PLAIN, 12);
    private final FontUIResource fUserFont = new FontUIResource("Arial", Font.PLAIN, 12);
    private final FontUIResource fSmallFont = new FontUIResource("Arial", Font.PLAIN, 10);
  }

// ---------------------------------------------------------------------------
  private static class Aqua extends Default {

    public String getName(){ return fName; }

    protected ColorUIResource getPrimary1() { return fPrimary1; }
    protected ColorUIResource getPrimary2() { return fPrimary2; }
    protected ColorUIResource getPrimary3() { return fPrimary3; }

    private final String fName = fAQUA_NAME;
    private final ColorUIResource fPrimary1 = new ColorUIResource(102, 153, 153);
    private final ColorUIResource fPrimary2 = new ColorUIResource(128, 192, 192);
    private final ColorUIResource fPrimary3 = new ColorUIResource(159, 235, 235);
  }

  private static class LargeFont extends Default {

    public String getName(){ return fName; }

    //fonts are larger than defaults
    public FontUIResource getControlTextFont() { return fControlFont;}
    public FontUIResource getSystemTextFont() { return fSystemFont;}
    public FontUIResource getUserTextFont() { return fUserFont;}
    public FontUIResource getMenuTextFont() { return fControlFont;}
    public FontUIResource getWindowTitleFont() { return fWindowTitleFont;}
    public FontUIResource getSubTextFont() { return fSmallFont;}

    private final String fName = fLARGE_FONT_NAME;

    private final FontUIResource fControlFont = new FontUIResource("Dialog", Font.BOLD, 18);
    private final FontUIResource fSystemFont = new FontUIResource("Dialog", Font.PLAIN, 18);
    private final FontUIResource fWindowTitleFont = new FontUIResource("Dialog", Font.BOLD,
                                                                        18);
    private final FontUIResource fUserFont = new FontUIResource("SansSerif", Font.PLAIN, 18);
    private final FontUIResource fSmallFont = new FontUIResource("Dialog", Font.PLAIN, 14);
  }

  private static class LowVision extends LargeFont {
    public String getName() { return fName; }

    //colors are mostly black and white
    public ColorUIResource getPrimaryControlHighlight() { return fPrimaryHighlight;}
    public ColorUIResource getControlHighlight() { return super.getSecondary3(); }
    public ColorUIResource getFocusColor() { return getBlack(); }
    public ColorUIResource getTextHighlightColor() { return getBlack(); }
    public ColorUIResource getHighlightedTextColor() { return getWhite(); }
    public ColorUIResource getMenuSelectedBackground() { return getBlack(); }
    public ColorUIResource getMenuSelectedForeground() { return getWhite(); }
    public ColorUIResource getAcceleratorForeground() { return getBlack(); }
    public ColorUIResource getAcceleratorSelectedForeground() { return getWhite(); }

    public void addCustomEntriesToTable(UIDefaults aTable) {
      super.addCustomEntriesToTable(aTable);

      aTable.put( "ToolTip.border", fBlackLineBorder);
      aTable.put( "TitledBorder.border", fBlackLineBorder);
      aTable.put( "ScrollPane.border", fBlackLineBorder);

      aTable.put( "TextField.border", fTextBorder);
      aTable.put( "PasswordField.border", fTextBorder);
      aTable.put( "TextArea.border", fTextBorder);
      aTable.put( "TextPane.border", fTextBorder);
      aTable.put( "EditorPane.border", fTextBorder);

      aTable.put("InternalFrame.closeIcon",
                 MetalIconFactory.getInternalFrameCloseIcon(fInternalFrameIconSize));
      aTable.put("InternalFrame.maximizeIcon",
                  MetalIconFactory.getInternalFrameMaximizeIcon(fInternalFrameIconSize));
      aTable.put("InternalFrame.iconifyIcon",
                  MetalIconFactory.getInternalFrameMinimizeIcon(fInternalFrameIconSize));
      aTable.put("InternalFrame.minimizeIcon",
                  MetalIconFactory.getInternalFrameAltMaximizeIcon(fInternalFrameIconSize));

      aTable.put( "ScrollBar.width", fScrollBarWidth );
    }

    protected ColorUIResource getPrimary1() { return fPrimary1; }
    protected ColorUIResource getPrimary2() { return fPrimary2; }
    protected ColorUIResource getPrimary3() { return fPrimary3; }
    protected ColorUIResource getSecondary2() { return fSecondary2; }
    protected ColorUIResource getSecondary3() { return fSecondary3; }

    private static final String fName = fLOW_VISION_NAME;

    private final ColorUIResource fPrimary1 = new ColorUIResource(0, 0, 0);
    private final ColorUIResource fPrimary2 = new ColorUIResource(204, 204, 204);
    private final ColorUIResource fPrimary3 = new ColorUIResource(255, 255, 255);
    private final ColorUIResource fPrimaryHighlight = new ColorUIResource(102,102,102);
    private final ColorUIResource fSecondary2 = new ColorUIResource(204, 204, 204);
    private final ColorUIResource fSecondary3 = new ColorUIResource(255, 255, 255);
    private final ColorUIResource fControlHighlight = new ColorUIResource(102,102,102);

    private final Border fBlackLineBorder = new BorderUIResource(new LineBorder(getBlack()));
    private final Object fTextBorder = new BorderUIResource(
                                       new CompoundBorder(
                                       fBlackLineBorder, new BasicBorders.MarginBorder()));

    private final int fInternalFrameIconSize = 30;

    private final Integer fScrollBarWidth = new Integer(25);
  }
}
